{#
    basic/layout.html
    ~~~~~~~~~~~~~~~~~

    Master layout template for Sphinx themes.

    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.
#}
{%- block doctype -%}
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
{%- endblock %}
{%- set reldelim1 = reldelim1 is not defined and ' &raquo;' or reldelim1 %}
{%- set reldelim2 = reldelim2 is not defined and ' |' or reldelim2 %}
{%- set render_sidebar = (not embedded) and (not theme_nosidebar|tobool) and
                         (sidebars != []) %}
{%- set url_root = pathto('', 1) %}
{# XXX necessary? #}
{%- if url_root == '#' %}{% set url_root = '' %}{% endif %}
{%- if not embedded and docstitle %}
  {%- set titlesuffix = " &mdash; "|safe + docstitle|e %}
{%- else %}
  {%- set titlesuffix = "" %}
{%- endif %}

{%- macro relbar() %}
    <div class="related">
      <h3>{{ _('Navigation') }}</h3>
      <ul>
        {%- for rellink in rellinks %}
        <li class="right" {% if loop.first %}style="margin-right: 10px"{% endif %}>
          <a href="{{ pathto(rellink[0]) }}" title="{{ rellink[1]|striptags|e }}"
             {{ accesskey(rellink[2]) }}>{{ rellink[3] }}</a>
          {%- if not loop.first %}{{ reldelim2 }}{% endif %}</li>
        {%- endfor %}
        {%- block rootrellink %}
        <li><a href="{{ pathto(master_doc) }}">{{ shorttitle|e }}</a>{{ reldelim1 }}</li>
        {%- endblock %}
        {%- for parent in parents %}
          <li><a href="{{ parent.link|e }}" {% if loop.last %}{{ accesskey("U") }}{% endif %}>{{ parent.title }}</a>{{ reldelim1 }}</li>
        {%- endfor %}
        {%- block relbaritems %} {% endblock %}
      </ul>
    </div>
{%- endmacro %}

{%- macro sidebar() %}
      {%- if render_sidebar %}
      <div class="sphinxsidebar">
        <div class="sphinxsidebarwrapper">

				<div class="side-column-block"><a href="http://www.percona.com/software/percona-toolkit/"><img alt="Percona Toolkit" src="http://s2.percona.com/logo_percona_toolkit_new.png" /></a></div>

			
			<!-- div class="percona_com_ad" style="width:220px;"><script type="text/javascript" src="http://www.percona.com/ads.php?size=220x120"></script></div -->
							
          {%- if sidebars != None %}
            {#- new style sidebar: explicitly include/exclude templates #}
            {%- for sidebartemplate in sidebars %}
            {%- include sidebartemplate %}
            {%- endfor %}
          {%- else %}
            {#- old style sidebars: using blocks -- should be deprecated #}
			            {%- block serverseries %}
		<div class="side-column-block">
			<h3>Percona Toolkit Series</h3>
			     <ul><li><a href="http://www.percona.com/doc/percona-toolkit/1.0/">Percona Toolkit 1.0</a></li><li><a href="http://www.percona.com/doc/percona-toolkit/2.0/">Percona Toolkit 2.0</a></li><li><a href="http://www.percona.com/doc/percona-toolkit/2.1/">Percona Toolkit 2.1</a></li><li><a href="http://www.percona.com/doc/percona-toolkit/2.2/">Percona Toolkit 2.2</a></li></ul>
			</div>
			            {%- endblock %}
            {%- block sidebartoc %}
            {%- include "localtoc.html" %}
            {%- endblock %}
            {%- block sidebarrel %}
            {%- include "relations.html" %}
            {%- endblock %}
            {%- block sidebarsourcelink %}
            {%- include "sourcelink.html" %}
            {%- endblock %}
            {%- if customsidebar %}
            {%- include customsidebar %}
            {%- endif %}
            {%- block sidebarsearch %}
            {%- include "searchbox.html" %}
            {%- endblock %}
          {%- endif %}

			
        </div>
      </div>
      {%- endif %}
{%- endmacro %}

{%- macro script() %}
    <script type="text/javascript">
      var DOCUMENTATION_OPTIONS = {
        URL_ROOT:    '{{ url_root }}',
        VERSION:     '{{ release|e }}',
        COLLAPSE_INDEX: false,
        FILE_SUFFIX: '{{ '' if no_search_suffix else file_suffix }}',
        HAS_SOURCE:  {{ has_source|lower }}
      };
    </script>
    {%- for scriptfile in script_files %}
    <script type="text/javascript" src="{{ pathto(scriptfile, 1) }}"></script>
    {%- endfor %}
{%- endmacro %}

{%- macro css() %}
    <link rel="stylesheet" href="{{ pathto('_static/percona.com.css', 1) }}" type="text/css" />
    <link rel="stylesheet" href="{{ pathto('_static/' + style, 1) }}" type="text/css" />
    <link rel="stylesheet" href="{{ pathto('_static/pygments.css', 1) }}" type="text/css" />
    {%- for cssfile in css_files %}
    <link rel="stylesheet" href="{{ pathto(cssfile, 1) }}" type="text/css" />
    {%- endfor %}
<link rel="stylesheet" href="//www.percona.com/sites/all/themes/percona/css/onlinedoc-responsive.css" type="text/css" />

{%- endmacro %}

<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <meta http-equiv="Content-Type" content="text/html; charset={{ encoding }}" />
    <link rel="shortcut icon" href="_static/percona_favicon.ico"/>
    {{ metatags }}
    {%- block htmltitle %}
    <title>{{ title|striptags|e }}{{ titlesuffix }}</title>
    {%- endblock %}

    {{ css() }}
    {%- if not embedded %}
    {{ script() }}
    {%- if use_opensearch %}
    <link rel="search" type="application/opensearchdescription+xml"
          title="{% trans docstitle=docstitle|e %}Search within {{ docstitle }}{% endtrans %}"
          href="{{ pathto('_static/opensearch.xml', 1) }}"/>
    {%- endif %}

<script src="{{ pathto('_static/percona.com.js', 1)}}" type="text/javascript"></script> 

    {%- endif %}
{%- block linktags %}
    {%- if hasdoc('about') %}
    <link rel="author" title="{{ _('About these documents') }}" href="{{ pathto('about') }}" />
    {%- endif %}
    {%- if hasdoc('genindex') %}
    <link rel="index" title="{{ _('Index') }}" href="{{ pathto('genindex') }}" />
    {%- endif %}
    {%- if hasdoc('search') %}
    <link rel="search" title="{{ _('Search') }}" href="{{ pathto('search') }}" />
    {%- endif %}
    {%- if hasdoc('copyright') %}
    <link rel="copyright" title="{{ _('Copyright') }}" href="{{ pathto('copyright') }}" />
    {%- endif %}
    <link rel="top" title="{{ docstitle|e }}" href="{{ pathto('index') }}" />
    {%- if parents %}
    <link rel="up" title="{{ parents[-1].title|striptags|e }}" href="{{ parents[-1].link|e }}" />
    {%- endif %}
    {%- if next %}
    <link rel="next" title="{{ next.title|striptags|e }}" href="{{ next.link|e }}" />
    {%- endif %}
    {%- if prev %}
    <link rel="prev" title="{{ prev.title|striptags|e }}" href="{{ prev.link|e }}" />
    {%- endif %}
{%- endblock %}
{%- block extrahead %} {% endblock %}
  <link href="//maxcdn.bootstrapcdn.com/font-awesome/4.2.0/css/font-awesome.min.css" rel="stylesheet">
  </head>
  <body>

<div id="wrapper">
      <div id="stickycontent">

{%- block header %}
  

    
        <header id="header">
          <div id="headerContent">
            <div class="topNavContainer">
              <div class="logo" itemscope="" itemtype="http://schema.org/ImageObject" style="float:left;width:230px;">
                <a href="/"><img src="/sites/all/themes/percona/logo.png" alt="Percona" itemprop="url" height="66" width="220"></a>
              </div>
              <div id="topnav">
                <a href="http://www.percona.com/blog/">BLOG</a> / <a href="http://www.percona.com/forums">FORUM</a> / <a href="/">ONLINE TOOLS</a>  /  <a href="http://www.percona.com/live/">PERCONA LIVE</a>  /  <a href="http://customer.percona.com">SUPPORT PORTAL</a>  /  <a href="https://www.percona.com/contact/sales">CONTACT</a>
                <div class="search-field">
                  <form id="search-form" action="http://www.percona.com/search" method="get">
                    <div class="form">
                      <input id="search-input" maxlength="100" name="s" type="text">
                      <input value="search" class="search-button" type="submit">
                    </div>
                  </form>
                </div>
              </div>
                <a href="/support"><div class="getHelpBox">GET 24/7 LIVE HELP NOW <img src="/sites/all/themes/percona/images/right-white-arrow.png" class="getHelpArrow"></div></a>
              </div>
            <div style="clear:both;"></div>
          </div>
          <nav>
            <div class="tb-megamenu tb-megamenu-main-menu">
              <div class="nav-collapse collapse always-show">
                  <ul class="tb-megamenu-nav nav level-0 items-11">
                    <li class="tb-megamenu-item level-1 mega mega-align-left dropdown">
                      <a href="http://www.percona.com/services" class="dropdown-toggle">Services</a>
                    </li>

                  <li class="tb-megamenu-item level-1 mega dropdown">
                      <a href="http://www.percona.com/software" class="dropdown-toggle">Software</a>
                  </li>

                  <li class="tb-megamenu-item level-1 mega dropdown">
                      <a href="http://www.percona.com/resources" class="dropdown-toggle">Resources</a>
                  </li>

                  <li class="tb-megamenu-item level-1 mega dropdown">
                      <a href="http://www.percona.com/news-and-events" class="dropdown-toggle">News &amp; Events</a>
                  </li>

                  <li class="tb-megamenu-item level-1 mega mega-align-left dropdown">
                      <a href="http://www.percona.com/percona-community" class="dropdown-toggle">Community</a>
                  </li>

                  <li class="tb-megamenu-item level-1 mega mega-align-right dropdown">
                      <a href="http://www.percona.com/about-percona" class="dropdown-toggle">About</a>
                  </li>
                </ul>
                </div>
            </div>
          </nav> 
        </header>
        


{% endblock %}

      <!-- <div id="stripe"></div> -->

{%- block content %}

<div class="fixedwidth-wrapper">

  {%- block sidebar1 %} {# possible location for sidebar #} {% endblock %}

    <div class="document">
      


  {%- block document %}
      <div class="documentwrapper">
      {%- if render_sidebar %}
        <div class="bodywrapper">
      {%- endif %}

        {%- block relbar1 %}{{ relbar() }}{% endblock %}

          <div class="body">

            <div class="cta-sidebar-second">

                <div class="side-column-block">
				<div class="a-btn-container">
					<a title="Download PDF Manual for Percona Toolkit 2.2" onclick="_gaq.push(['b._trackEvent', 'Percona Toolkit', 'Download', 'Download Manual Percona Toolkit 2.2 ']);" href="http://form.percona.com/Percona-Toolkit-22-Series.html" class="a-btn-green">
						<span class="a-btn-text-green">Download Manual</span>
						<span class="a-btn-fixed-slide-text-green">PDF for version 2.2</span>
						<span class="a-btn-icon-right-green"><span></span></span>
					</a>
				</div></div>

                  <div class="side-column-block" id="block-cta-buttons">
                      <div class="content">
                      <a href="http://form.percona.com/ContactMe.html" title="Contact Me">
                          <div class="CTA-buttons">
                            <span class="a-btn-text-orange"><i class="fa fa-user fa-2" style="padding-right:5px;"></i>CONTACT ME</span><span></span>  
                          </div>
                        </a>

                      <a href="/webinars" title="Attend a Webinar"><div class="CTA-buttons">
                      <span class="a-btn-text-orange"><i class="fa fa-desktop fa-3" style="padding-right:5px;"></i>ATTEND A WEBINAR</span>  
                      </div></a>

                      <a href="/downloads" title="Download Software"><div class="CTA-buttons">
                      <span class="a-btn-text-orange"><i class="fa fa-download fa-2" style="padding-right:5px;"></i>DOWNLOAD SOFTWARE</span>  
                      </div></a>

                      <a href="/subscribe" title="Register For Newsletters"><div class="CTA-buttons">
                      <span class="a-btn-text-orange"><i class="fa fa-pencil-square-o  fa-2" style="padding-right:5px;"></i>NEWSLETTER REGISTRATION</span>  
                      </div></a>

                    </div>
                </div>
          </div>

            {% block body %} {% endblock %}
          </div>

          {%- block relbar2 %}{{ relbar() }}

      {%- if render_sidebar %}
      <div class="percona_com_ad" style="margin:auto;width:630px;"><script type="text/javascript" src="http://www.percona.com/ads.php?size=630x80"></script></div>
        </div>
      {%- endif %}



      </div>

  {%- endblock %}

  {%- block sidebar2 %}{{ sidebar() }}{% endblock %}

      <div class="clearer"></div>

      {%- if last_updated %}
      {% trans last_updated=last_updated|e %}Last updated on {{ last_updated }}.{% endtrans %}
      {%- endif %}
     
     
    
    
    <div class="license">

      {%- if show_copyright %}
      {%- if hasdoc('copyright') %}
      {% trans path=pathto('copyright'), copyright=copyright|e %}&copy; <a href="{{ path }}">Copyright</a> {{ copyright }}.{% endtrans %}
      {%- else %}
      {% trans copyright=copyright|e %}&copy; Copyright {{ copyright }}.{% endtrans %}
      {%- endif %}
      {%- endif %}

      <br />
        Except where otherwise noted, this documentation is licensed under the following license:
      <br />
      <a class="urlextern" rel="license" href="http://creativecommons.org/licenses/by-sa/2.0/">
        CC Attribution-ShareAlike 2.0 Generic</a><br />

 {%- if show_sphinx %}
      {% trans sphinx_version=sphinx_version|e %}Created using <a href="http://sphinx.pocoo.org/">Sphinx</a> {{ sphinx_version }}.{% endtrans %}
    {%- endif %}    

     </div>

    </div>
    {%- endblock %}
</div>

{%- block footer %}


    <div class="file-bugs">
        This documentation is developed in Launchpad as part of the <a  rel="nofollow" href="https://code.launchpad.net/percona-toolkit">Percona Toolkit source code</a>.<br/>
    If you spotted innacuracies, errors, don't understood it or you think something is missing or should be improved, please <a rel="nofollow" href="https://bugs.launchpad.net/percona-toolkit/+filebug?field.tags=doc&amp;field.comment=[In:Percona Toolkit Documentation]&amp;?field.tags=doc&amp;field.title=[DOC] ">file a bug</a>.
  
    </div>
    


    {% endblock %}



    


{%- endblock %}
</div>
<div id="stickyfooter"> 
      <footer id="footer">
        <div class="footer">
          <div class="region region-footer">
            <div id="block-block-33" class="block block-block contextual-links-region">
              <div class="content">
              <div class="footerContainer">
                <div class="footerNewsletterContainer"><img src="/sites/all/themes/percona/images/footer-logo.png" /><br />
                  <div class="footerNewsletterTitle">Sign-up for Percona Newsletters</div>
                  <a href="/subscribe"><div class="footerNewsletter">Register Now</div></a>
                  <div class="footerSocialIcon"><a rel="nofollow" href="https://www.facebook.com/Percona" target="_blank"><img src="/sites/all/themes/percona/images/footer-facebook2.jpg"></a></div>
                  <div class="footerSocialIcon"><a rel="nofollow" href="http://twitter.com/percona" target="_blank"><img src="/sites/all/themes/percona/images/footer-twitter2.jpg"></a></div>
                  <div class="footerSocialIcon"><a rel="nofollow" href="http://www.linkedin.com/groups?gid=2299033&amp;trk=hb_side_g" target="_blank"><img src="/sites/all/themes/percona/images/footer-linkedin2.jpg"></a></div>
                  <div class="footerSocialIcon"><a rel="nofollow" href="https://launchpad.net/percona-project" target="_blank"><img src="/sites/all/themes/percona/images/footer-launchpad.jpg"></a></div>
                  <div class="footerSocialIcon"><a href=""><img src="/sites/all/themes/percona/images/footer-email.jpg"></a></div>
                </div>
                <!-- Eof LOGO / SUBSCRIBE / SOCIAL LINKS -->
                <div class="footerLinksContainer">
                  <div class="footerLinksRow1Col1">
                    <div class="footerLinksTitle"><a href="/services">SERVICES</a></div>
                    <div>
                      <ul class="footerLinks">
                      <li><a href="/services/mysql-support">Support</a></li>
                      <li><a href="/services/percona-managed-services">Managed Services</a></li>
                      <li><a href="/services/mysql-consulting">Consulting</a></li>
                      <li><a href="/services/mysql-training">MySQL Training</a></li>
                      </ul>
                    </div>
                  </div>


                  <div class="footerLinksRow1Col2">
                    <div class="footerLinksTitle"><a href="/software">SOFTWARE</a></div>
                    <div>
                      <ul class="footerLinks">
                      <li><a href="/software/percona-server">Percona Server</a></li>
                      <li><a href="/software/percona-xtradb-cluster">Percona XtraDB Cluster</a></li>
                      <li><a href="/software/percona-xtradb-cluster">Percona XtraBackup</a></li>
                      <li><a href="/software/percona-toolkit">Percona Toolkit</a></li>
                      <li><a href="/software/documentation">Documentation</a></li>
                      </ul>
                    </div>
                  </div>

                  <div class="footerLinksRow1Col3">
                    <div class="footerLinksTitle"><a href="/resources">RESOURCES</a></div>
                    <div>
                    <ul class="footerLinks">
                    <li><a href="/resources/mysql-webinars">Webinars</a></li>
                    <li><a href="https://tools.percona.com/wizard">MySQL Configuration Wizard</a></li>
                    <li><a href="/resources/technical-presentations">Technical Presentations</a></li>
                    <li><a href="/resources/mysql-videos">Videos</a></li>
                    <li><a href="/resources/mysql-ebooks">eBooks</a></li>
                    </ul>
                    </div>
                  </div>


        <!-- ROW 2-->
                  <div class="footerLinksRow2Col4">
                  <div class="footerLinksTitle"><a href="/percona-community">COMMUNITY</a></div>
                  <div>
                  <ul class="footerLinks">
                  <li><a href="/forums">Forums</a></li>
                  <li><a href="/blog">Performance Blog</a></li>
                  <li><a rel="nofollow" href="http://groups.google.com/group/percona-discussion" target="_blank">Google Groups</a></li>
                  <li><a rel="nofollow" href="https://launchpad.net/percona-project" target="_blank">Launchpad</a></li>
                  </ul>
                  </div>
                  </div>

                  <div class="footerLinksRow2Col5">
                  <div class="footerLinksTitle"><a href="/live/conferences">CONFERENCES</a></div>
                  <div>
                  <ul class="footerLinks">
                  <li><a href="http://www.percona.com/live/london-2014/" target="_blank">Percona Live London 2014</a></li>
                  <li><a href="http://www.percona.com/live/mysql-conference-2015/" target="_blank">Percona Live Silicon Valley 2015</a></li>
                  <li><a href="http://www.percona.com/live/openstack-live-2015/" target="_blank">OpenStack Live 2015</a></li>
                  <li><a href="/news-and-events/mysql-events/" target="_blank">Other MySQL Events</a></li>
                  </ul>
                  </div>
                  </div>

                  <div class="footerLinksRow2Col6">
                  <div class="footerLinksTitle"><a href="/about-percona">ABOUT</a></div>
                  <div>
                  <ul class="footerLinks">
                  <li><a href="/about-us/our-mission">Mission</a></li>
                  <li><a href="/about-us/our-team">Team</a></li>
                  <li><a href="/about-us/careers">Careers</a></li>
                  <li><a href="/support">Contact</a></li>
                  </ul>
                  </div>
                  </div>

                </div>

        <!-- GET HELP BOX-->
                <div class="footerGetHelpContainer">
                  <div class="footerGetHelpEmergency">EMERGENCY?<div class="footerGetHelpNow">Get 24/7 Live Help Now!</div></div>
                  
                  <div class="footerPhoneNoContainer">
                  <div class="footerSalesRegionTitle">Sales North America</div> 
                  <div class="footerPhoneNo">(888) 316-9775 or (208) 473-2904</div>

                  <div class="footerSalesRegionTitle">Sales Europe</div>  
                  <div class="footerPhoneNo">
                  +44-208-133-0309 (UK)<br>
                  0-800-051-8984 (UK)<br>
                  0-800-181-0665 (GER)</div>

                  <div class="footerSalesRegionTitle">Training</div>  
                  <div class="footerPhoneNo">
                  (855) 55TRAIN or (925) 271-5054</div>
                  </div>
                  <!-- Eof GET HELP BOX-->

                </div>
              </div>
            </div>      
          </div>
        </div>
        </div><!-- End of: div class="footer" -->
      </footer>
    </div>
    <div class="copyright-container">
      <div class="copyright">MySQL and InnoDB are trademarks of Oracle Corp. Proudly running Percona Server Copyright © 2006-2014 Percona LLC</div>
    </div>  
</div>

<!-- GA //-->
  <script type="text/javascript">
  var _gaq = _gaq || [];
  _gaq.push(['b._setAccount', 'UA-343802-3']);
  _gaq.push(['b._setDomainName', 'percona.com']);
  _gaq.push(['b._trackPageview']);
  (function() {
  var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;
  ga.src = ('https:' == document.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';
  var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(ga, s);
  })();
</script>

<!-- /GA //-->
<!-- NAVI //-->
<script type="text/javascript">//<![CDATA[

window.jQuery('#search-form').bind('submit', function() { window.location.href = jQuery('#search-form').attr('action') + '?s=' + jQuery('#search-input').val(); return false; });


//]]></script>


  </body>
</html>



{% if theme_collapsiblesidebar|tobool %}
{% set script_files = script_files + ['_static/sidebar.js'] %}
{% endif %}
