{#
    basic/searchbox.html
    ~~~~~~~~~~~~~~~~~~~~

    Sphinx sidebar template: quick search box.

    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.
#}
{%- if pagename != "search" %}
<div id="searchbox" style="display: none; padding: 3px;">
  <h3>{{ _('Quick search') }}</h3>
    <form class="search" action="{{ pathto('search') }}" method="get">
      <input type="text" name="q" size="18" />
      <input type="submit" value="{{ _('Go') }}" />
      <input type="hidden" name="check_keywords" value="yes" />
      <input type="hidden" name="area" value="default" />
    </form>

</div>
<script type="text/javascript">$('#searchbox').show(0);</script>
{%- endif %}
