mysql> show create table checksum_test_7\G
*************************** 1. row ***************************
       Table: checksum_test_7
Create Table: CREATE TABLE `checksum_test_7` (
  `a` time NOT NULL,
  `b` int(11) default NULL,
  PRIMARY KEY  (`a`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1
1 row in set (0.00 sec)

mysql> notee
