DROP DATABASE IF EXISTS on_del_cas;
CREATE DATABASE on_del_cas;
USE on_del_cas;

CREATE TABLE parent (
  id INT NOT NULL AUTO_INCREMENT PRIMARY KEY
) ENGINE=InnoDB;

CREATE TABLE child1 (
  id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
  parent_id INT NOT NULL,
  FOREIGN KEY child1_ibfk1 (parent_id) REFERENCES parent (id) ON DELETE RESTRICT
) ENGINE=InnoDB;

CREATE TABLE child2 (
  id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
  parent_id INT NOT NULL,
  FOREIGN KEY child2_ibfk1 (parent_id) REFERENCES parent (id) ON DELETE CASCADE
) ENGINE=InnoDB;
