DROP DATABASE IF EXISTS test;
CREATE DATABASE test;
CREATE TABLE test.t1 (
  i INT,
  UNIQUE INDEX (i)
);
INSERT INTO test.t1 VALUES (1), (2), (3);
CREATE TABLE test.t2 (
  i INT,
  UNIQUE INDEX (i)
);
INSERT INTO test.t2 VALUES (1), (2), (3), (4);
