USE test;
DROP TABLE IF EXISTS `issue_37`;
CREATE TABLE `issue_37` (
   a INT DEFAULT 0,
   UNIQUE INDEX idx_a (a)
);
CREATE TRIGGER a_trig BEFORE INSERT ON issue_37 FOR EACH ROW SET @a = @a + NEW.a;
