# Reset all Performance Schema configuration

--source ./ps_setup_instruments_cleanup.inc

--source ./ps_setup_actors_cleanup.inc

--source ./ps_setup_consumers_cleanup.inc

--source ./ps_threads_cleanup.inc
