CREATE DATABASE IF NOT EXISTS foo001;
CREATE DATABASE IF NOT EXISTS foo002;
CREATE DATABASE IF NOT EXISTS foo003;
CREATE DATABASE IF NOT EXISTS foo004;
CREATE DATABASE IF NOT EXISTS foo005;
CREATE DATABASE IF NOT EXISTS foo006;
CREATE DATABASE IF NOT EXISTS foo007;
CREATE DATABASE IF NOT EXISTS foo008;
CREATE DATABASE IF NOT EXISTS foo009;
CREATE DATABASE IF NOT EXISTS foo010;
CREATE DATABASE IF NOT EXISTS foo011;
CREATE DATABASE IF NOT EXISTS foo012;
CREATE DATABASE IF NOT EXISTS foo013;
CREATE DATABASE IF NOT EXISTS foo014;
CREATE DATABASE IF NOT EXISTS foo015;
CREATE DATABASE IF NOT EXISTS foo016;
CREATE DATABASE IF NOT EXISTS foo017;
CREATE DATABASE IF NOT EXISTS foo018;
CREATE DATABASE IF NOT EXISTS foo019;
CREATE DATABASE IF NOT EXISTS foo020;
CREATE DATABASE IF NOT EXISTS foo021;
CREATE DATABASE IF NOT EXISTS foo022;
CREATE DATABASE IF NOT EXISTS foo023;
CREATE DATABASE IF NOT EXISTS foo024;
CREATE DATABASE IF NOT EXISTS foo025;
CREATE DATABASE IF NOT EXISTS foo026;
CREATE DATABASE IF NOT EXISTS foo027;
CREATE DATABASE IF NOT EXISTS foo028;
CREATE DATABASE IF NOT EXISTS foo029;
CREATE DATABASE IF NOT EXISTS foo030;
CREATE DATABASE IF NOT EXISTS foo031;
CREATE DATABASE IF NOT EXISTS foo032;
CREATE DATABASE IF NOT EXISTS foo033;
CREATE DATABASE IF NOT EXISTS foo034;
CREATE DATABASE IF NOT EXISTS foo035;
CREATE DATABASE IF NOT EXISTS foo036;
CREATE DATABASE IF NOT EXISTS foo037;
CREATE DATABASE IF NOT EXISTS foo038;
CREATE DATABASE IF NOT EXISTS foo039;
CREATE DATABASE IF NOT EXISTS foo040;
CREATE DATABASE IF NOT EXISTS foo041;
CREATE DATABASE IF NOT EXISTS foo042;
CREATE DATABASE IF NOT EXISTS foo043;
CREATE DATABASE IF NOT EXISTS foo044;
CREATE DATABASE IF NOT EXISTS foo045;
CREATE DATABASE IF NOT EXISTS foo046;
CREATE DATABASE IF NOT EXISTS foo047;
CREATE DATABASE IF NOT EXISTS foo048;
CREATE DATABASE IF NOT EXISTS foo049;
CREATE DATABASE IF NOT EXISTS foo050;
CREATE DATABASE IF NOT EXISTS foo051;
CREATE DATABASE IF NOT EXISTS foo052;
CREATE DATABASE IF NOT EXISTS foo053;
CREATE DATABASE IF NOT EXISTS foo054;
CREATE DATABASE IF NOT EXISTS foo055;
CREATE DATABASE IF NOT EXISTS foo056;
CREATE DATABASE IF NOT EXISTS foo057;
CREATE DATABASE IF NOT EXISTS foo058;
CREATE DATABASE IF NOT EXISTS foo059;
CREATE DATABASE IF NOT EXISTS foo060;
CREATE DATABASE IF NOT EXISTS foo061;
CREATE DATABASE IF NOT EXISTS foo062;
CREATE DATABASE IF NOT EXISTS foo063;
CREATE DATABASE IF NOT EXISTS foo064;
CREATE DATABASE IF NOT EXISTS foo065;
CREATE DATABASE IF NOT EXISTS foo066;
CREATE DATABASE IF NOT EXISTS foo067;
CREATE DATABASE IF NOT EXISTS foo068;
CREATE DATABASE IF NOT EXISTS foo069;
CREATE DATABASE IF NOT EXISTS foo070;
CREATE DATABASE IF NOT EXISTS foo071;
CREATE DATABASE IF NOT EXISTS foo072;
CREATE DATABASE IF NOT EXISTS foo073;
CREATE DATABASE IF NOT EXISTS foo074;
CREATE DATABASE IF NOT EXISTS foo075;
CREATE DATABASE IF NOT EXISTS foo076;
CREATE DATABASE IF NOT EXISTS foo077;
CREATE DATABASE IF NOT EXISTS foo078;
CREATE DATABASE IF NOT EXISTS foo079;
CREATE DATABASE IF NOT EXISTS foo080;
CREATE DATABASE IF NOT EXISTS foo081;
CREATE DATABASE IF NOT EXISTS foo082;
CREATE DATABASE IF NOT EXISTS foo083;
CREATE DATABASE IF NOT EXISTS foo084;
CREATE DATABASE IF NOT EXISTS foo085;
CREATE DATABASE IF NOT EXISTS foo086;
CREATE DATABASE IF NOT EXISTS foo087;
CREATE DATABASE IF NOT EXISTS foo088;
CREATE DATABASE IF NOT EXISTS foo089;
CREATE DATABASE IF NOT EXISTS foo090;
CREATE DATABASE IF NOT EXISTS foo091;
CREATE DATABASE IF NOT EXISTS foo092;
CREATE DATABASE IF NOT EXISTS foo093;
CREATE DATABASE IF NOT EXISTS foo094;
CREATE DATABASE IF NOT EXISTS foo095;
CREATE DATABASE IF NOT EXISTS foo096;
CREATE DATABASE IF NOT EXISTS foo097;
CREATE DATABASE IF NOT EXISTS foo098;
CREATE DATABASE IF NOT EXISTS foo099;
CREATE DATABASE IF NOT EXISTS foo100;
CREATE TABLE IF NOT EXISTS foo001.bar001 (
  `id` int unsigned NOT NULL AUTO_INCREMENT,
  `value` varchar(255) NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
