DROP DATABASE IF EXISTS foo001;
DROP DATABASE IF EXISTS foo002;
DROP DATABASE IF EXISTS foo003;
DROP DATABASE IF EXISTS foo004;
DROP DATABASE IF EXISTS foo005;
DROP DATABASE IF EXISTS foo006;
DROP DATABASE IF EXISTS foo007;
DROP DATABASE IF EXISTS foo008;
DROP DATABASE IF EXISTS foo009;
DROP DATABASE IF EXISTS foo010;
DROP DATABASE IF EXISTS foo011;
DROP DATABASE IF EXISTS foo012;
DROP DATABASE IF EXISTS foo013;
DROP DATABASE IF EXISTS foo014;
DROP DATABASE IF EXISTS foo015;
DROP DATABASE IF EXISTS foo016;
DROP DATABASE IF EXISTS foo017;
DROP DATABASE IF EXISTS foo018;
DROP DATABASE IF EXISTS foo019;
DROP DATABASE IF EXISTS foo020;
DROP DATABASE IF EXISTS foo021;
DROP DATABASE IF EXISTS foo022;
DROP DATABASE IF EXISTS foo023;
DROP DATABASE IF EXISTS foo024;
DROP DATABASE IF EXISTS foo025;
DROP DATABASE IF EXISTS foo026;
DROP DATABASE IF EXISTS foo027;
DROP DATABASE IF EXISTS foo028;
DROP DATABASE IF EXISTS foo029;
DROP DATABASE IF EXISTS foo030;
DROP DATABASE IF EXISTS foo031;
DROP DATABASE IF EXISTS foo032;
DROP DATABASE IF EXISTS foo033;
DROP DATABASE IF EXISTS foo034;
DROP DATABASE IF EXISTS foo035;
DROP DATABASE IF EXISTS foo036;
DROP DATABASE IF EXISTS foo037;
DROP DATABASE IF EXISTS foo038;
DROP DATABASE IF EXISTS foo039;
DROP DATABASE IF EXISTS foo040;
DROP DATABASE IF EXISTS foo041;
DROP DATABASE IF EXISTS foo042;
DROP DATABASE IF EXISTS foo043;
DROP DATABASE IF EXISTS foo044;
DROP DATABASE IF EXISTS foo045;
DROP DATABASE IF EXISTS foo046;
DROP DATABASE IF EXISTS foo047;
DROP DATABASE IF EXISTS foo048;
DROP DATABASE IF EXISTS foo049;
DROP DATABASE IF EXISTS foo050;
DROP DATABASE IF EXISTS foo051;
DROP DATABASE IF EXISTS foo052;
DROP DATABASE IF EXISTS foo053;
DROP DATABASE IF EXISTS foo054;
DROP DATABASE IF EXISTS foo055;
DROP DATABASE IF EXISTS foo056;
DROP DATABASE IF EXISTS foo057;
DROP DATABASE IF EXISTS foo058;
DROP DATABASE IF EXISTS foo059;
DROP DATABASE IF EXISTS foo060;
DROP DATABASE IF EXISTS foo061;
DROP DATABASE IF EXISTS foo062;
DROP DATABASE IF EXISTS foo063;
DROP DATABASE IF EXISTS foo064;
DROP DATABASE IF EXISTS foo065;
DROP DATABASE IF EXISTS foo066;
DROP DATABASE IF EXISTS foo067;
DROP DATABASE IF EXISTS foo068;
DROP DATABASE IF EXISTS foo069;
DROP DATABASE IF EXISTS foo070;
DROP DATABASE IF EXISTS foo071;
DROP DATABASE IF EXISTS foo072;
DROP DATABASE IF EXISTS foo073;
DROP DATABASE IF EXISTS foo074;
DROP DATABASE IF EXISTS foo075;
DROP DATABASE IF EXISTS foo076;
DROP DATABASE IF EXISTS foo077;
DROP DATABASE IF EXISTS foo078;
DROP DATABASE IF EXISTS foo079;
DROP DATABASE IF EXISTS foo080;
DROP DATABASE IF EXISTS foo081;
DROP DATABASE IF EXISTS foo082;
DROP DATABASE IF EXISTS foo083;
DROP DATABASE IF EXISTS foo084;
DROP DATABASE IF EXISTS foo085;
DROP DATABASE IF EXISTS foo086;
DROP DATABASE IF EXISTS foo087;
DROP DATABASE IF EXISTS foo088;
DROP DATABASE IF EXISTS foo089;
DROP DATABASE IF EXISTS foo090;
DROP DATABASE IF EXISTS foo091;
DROP DATABASE IF EXISTS foo092;
DROP DATABASE IF EXISTS foo093;
DROP DATABASE IF EXISTS foo094;
DROP DATABASE IF EXISTS foo095;
DROP DATABASE IF EXISTS foo096;
DROP DATABASE IF EXISTS foo097;
DROP DATABASE IF EXISTS foo098;
DROP DATABASE IF EXISTS foo099;
DROP DATABASE IF EXISTS foo100;
