CREATE SCHEMA IF NOT EXISTS bug_1592608;
USE bug_1592608;
DROP TABLE IF EXISTS t;
CREATE TABLE t (
  id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY, 
  a  MEDIUMTEXT, 
  b  MEDIUMBLOB
)engine=innodb;

INSERT INTO t (a, b) VALUES (REPEAT('a', 2097152*2), CAST(REPEAT('a', 2097152*2) AS BINARY));

