CREATE TABLE `geodb_coordinates` (
  `loc_id` int(11) NOT NULL default '0',
  `lon` double default NULL,
  `lat` double default NULL,
  `sin_lon` double default NULL,
  `sin_lat` double default NULL,
  `cos_lon` double default NULL,
  `cos_lat` double default NULL,
  `coord_type` int(11) NOT NULL default '0',
  `coord_subtype` int(11) default NULL,
  `valid_since` date default NULL,
  `date_type_since` int(11) default NULL,
  `valid_until` date NOT NULL default '0000-00-00',
  `date_type_until` int(11) NOT NULL default '0',
  KEY `coord_loc_id_idx` (`loc_id`),
  KEY `coord_lon_idx` (`lon`),
  KEY `coord_lat_idx` (`lat`),
  KEY `coord_slon_idx` (`sin_lon`),
  KEY `coord_slat_idx` (`sin_lat`),
  KEY `coord_clon_idx` (`cos_lon`),
  KEY `coord_clat_idx` (`cos_lat`),
  KEY `coord_type_idx` (`coord_type`),
  KEY `coord_stype_idx` (`coord_subtype`),
  KEY `coord_since_idx` (`valid_since`),
  KEY `coord_until_idx` (`valid_until`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1