DROP DATABASE IF EXISTS test;
CREATE DATABASE test;
USE test;


CREATE TABLE t3 (
    c INT PRIMARY KEY AUTO_INCREMENT
) ENGINE=InnoDB PARTITION BY HASH(c) PARTITIONS 4;

INSERT INTO t3 VALUES (1), (2), (3), (4), (5);
